@@ -1,6 +1,8 @@
 # Allow use of TA_ env vars instead of HT_
-from hypertrace import env_var_settings
+import os
 
+import psutil
+from hypertrace import env_var_settings
 from traceableai.agent import Agent
 
 env_var_settings.ENV_VAR_PREFIXES.append('TA')
@@ -16,7 +18,19 @@ from hypertrace.agent.custom_logger import get_custom_logger  # pylint:disable=C
 
 config = Config()
 logger = get_custom_logger(__name__)
-if config.config.blocking_config.enabled.value is True:
+
+__POST_INIT = False
+POST_FORK_SERVERS = ['gunicorn']
+
+original_process = psutil.Process(os.getpid())
+args = original_process.cmdline()
+for entry in POST_FORK_SERVERS:
+    for arg in args:
+        if entry in arg:
+            __POST_INIT = True
+            logger.info('Detected server %s - deferring filter loading until post fork', entry)
+            break
+
+if __POST_INIT is not True:
+    logger.info("Adding TraceableAI Filter during autoinstrumentation")
     a.add_traceable_filter()
-else:
-    logger.info("Not adding traceableai blocking module")
