@@ -3,15 +3,18 @@ import traceback
 import logging
 
 import hypertrace.agent.filter
+from hypertrace.agent.filter.registry import TYPE_HTTP, TYPE_RPC
 from opentelemetry.trace import Span
 from traceableai.config.config import Config
 
 logger = logging.getLogger(__name__)
 
 _BLOCKING_AVAILABLE = False
+_NOT_BODY = False
+_IS_BODY = True
 
 try:
-    import traceableai.filter._blocking as _blocking # pylint: disable=import-error,no-name-in-module
+    import traceableai.filter._blocking as _blocking # pylint: disable=E0611,E0401,bad-option-value
     _BLOCKING_AVAILABLE = True
 except Exception as error:  # pylint: disable=broad-except
     _BLOCKING_AVAILABLE = False
@@ -19,7 +22,17 @@ except Exception as error:  # pylint: disable=broad-except
     logger.warning(error)
     logger.warning("error loading traceable extension - blocking filter disabled")
 
-HTTP_HEADER_PREFIX = "http.request.header"
+
+_HEADER_PREFIXES = {
+    TYPE_HTTP: 'http.request.header',
+    TYPE_RPC: 'rpc.request.metadata'
+}
+
+_BODY_PREFIXES = {
+    TYPE_HTTP: 'http.request.body',
+    TYPE_RPC: 'rpc.request.body'
+}
+
 
 class Traceable(hypertrace.agent.filter.Filter):
 
@@ -39,27 +52,16 @@ class Traceable(hypertrace.agent.filter.Filter):
         else:
             self.blocking_available = False
 
-    def evaluate_url_and_headers(self, span: Span, url: str, headers: dict) -> bool:
+    def evaluate_url_and_headers(self, span: Span, url: str, headers: dict, request_type) -> bool:
         if not self.blocking_available:
             return False
 
-        pairs = []
-
-        ip_address = span._readable_span().attributes.get('net.peer.ip') # pylint:disable=W0212
-        if ip_address:
-            pairs.append((_blocking.ffi.new("char[]", b"net.peer.ip"),
-                          _blocking.ffi.new("char[]", f"{ip_address}".encode('ascii'))))
-
-
+        pairs = _add_header_and_span_attributes(span, headers, request_type)
 
         # add url as part of header processing
         pairs.append((_blocking.ffi.new("char[]", b"http.url"),
                       _blocking.ffi.new("char[]", f"{url}".encode('ascii'))))
 
-        for header_key, header_value in headers.items():
-            pairs.append((_blocking.ffi.new("char[]", f"{HTTP_HEADER_PREFIX}.{header_key}".lower().encode('ascii')),
-                          _blocking.ffi.new("char[]", f"{header_value}".encode('ascii'))))
-
         num_headers = len(pairs)
         attributes = _blocking.ffi.new("traceable_attributes*")
         attribute_array = _blocking.ffi.new("traceable_attribute[]", num_headers)
@@ -70,28 +72,41 @@ class Traceable(hypertrace.agent.filter.Filter):
             attribute_array[index].value = pair[1]
         attributes.attribute_array = attribute_array
 
-        return self._evaluate_attributes(span, attributes)
+        return self._evaluate_attributes(span, attributes, _NOT_BODY)
 
-    def evaluate_body(self, span: Span, body) -> bool:
+    def evaluate_body(self, span: Span, body, headers: dict, request_type) -> bool:
         if not self.blocking_available:
             return False
 
         if not body or len(str(body)) == 0:
             return False
 
-        attributes = _blocking.ffi.new("traceable_attributes*")
-        attribute_array = _blocking.ffi.new("traceable_attribute[]", 1)
-        attributes.count = 1
-        attribute_array[0].key = _blocking.ffi.new("char[]", b"http.request.body")
-        attribute_array[0].value = _blocking.ffi.new("char[]", f"{body}".encode('ascii'))
+        pairs = _add_header_and_span_attributes(span, headers, request_type)
+        prefix = _BODY_PREFIXES[request_type]
+        pairs.append((_blocking.ffi.new("char[]", prefix.encode('ascii')),
+                      _blocking.ffi.new("char[]", f"{body}".encode('ascii'))))
 
+        num_attrs = len(pairs)
+        attributes = _blocking.ffi.new("traceable_attributes*")
+        attribute_array = _blocking.ffi.new("traceable_attribute[]", num_attrs)
+        attributes.count = num_attrs
+
+        for index, pair in enumerate(pairs, start=0):
+            attribute_array[index].key = pair[0]
+            attribute_array[index].value = pair[1]
         attributes.attribute_array = attribute_array
 
-        return self._evaluate_attributes(span, attributes)
 
-    def _evaluate_attributes(self, span, attributes) -> bool:
+        return self._evaluate_attributes(span, attributes, _IS_BODY)
+
+    def _evaluate_attributes(self, span, attributes, is_body) -> bool:
         blocking_result = _blocking.ffi.new("traceable_block_result*")
-        result_code = _blocking.lib.traceable_block_request(self.blocking_engine, attributes[0], blocking_result)
+        result_code = _blocking.lib.TRACEABLE_SUCCESS
+        if is_body:
+            result_code = _blocking.lib.traceable_block_request_body(self.blocking_engine, attributes[0], blocking_result) # pylint:disable=C0301
+        else:
+            result_code = _blocking.lib.traceable_block_request(self.blocking_engine, attributes[0], blocking_result)
+
         if result_code != _blocking.lib.TRACEABLE_SUCCESS:
             logger.debug ("traceable_block_request fail!")
             return False
@@ -117,11 +132,28 @@ def _add_span_attributes(span: Span, blocking_result):
 
         span.set_attribute(key.decode('utf-8', 'backslashescape'), value.decode('utf-8', 'backslashescape'))
 
+def _add_header_and_span_attributes(span: Span, headers: dict, request_type) -> []:
+    pairs = []
+    ip_address = span._readable_span().attributes.get('net.peer.ip')  # pylint:disable=W0212
+    if ip_address:
+        pairs.append((_blocking.ffi.new("char[]", b"net.peer.ip"),
+                      _blocking.ffi.new("char[]", f"{ip_address}".encode('ascii'))))
+
+    header_prefix = _HEADER_PREFIXES[request_type]
+    for header_key, header_value in headers.items():
+        pairs.append((_blocking.ffi.new("char[]", f"{header_prefix}.{header_key}".lower().encode('ascii')),
+                      _blocking.ffi.new("char[]", f"{header_value}".encode('ascii'))))
+
+    return pairs
+
 
 def _create_blocking_config():
     traceable_config = Config().config
-    should_debug = 1 if os.environ.get("TA_LOG_LEVEL", "").upper() == "DEBUG" else 0
-    should_debug = 1 if should_debug or traceable_config.blocking_config.debug_log else 0
+    should_debug = 0
+    if os.environ.get("TA_LOG_LEVEL", "").upper() == "DEBUG":
+        should_debug = 1
+    if traceable_config.blocking_config.debug_log.value is True:
+        should_debug = 1
 
     blocking_config = _blocking.ffi.new("traceable_blocking_config*")
 
@@ -138,6 +170,7 @@ def _create_blocking_config():
     blocking_config.opa_config.max_delay = traceable_opa.poll_period_seconds.value
     blocking_config.opa_config.logging_dir = _blocking.ffi.new("char[]", "".encode('ascii'))
     blocking_config.opa_config.logging_file_prefix = _blocking.ffi.new("char[]", "".encode('ascii'))
+    blocking_config.opa_config.cert_file = _blocking.ffi.new("char[]", "".encode('ascii'))
 
     modsecurity_enabled = 1 if traceable_config.blocking_config.modsecurity.enabled.value is True else 0
     blocking_config.modsecurity_config.enabled = modsecurity_enabled
@@ -148,6 +181,9 @@ def _create_blocking_config():
     remote_config_enabled = 1 if traceable_config.blocking_config.remote_config.enabled.value is True else 0
     blocking_config.remote_config.enabled = remote_config_enabled
 
+    cert_file = _blocking.ffi.new("char[]", "".encode('ascii'))
+    blocking_config.remote_config.cert_file = cert_file
+
     remote_config_endpoint = traceable_config.blocking_config.remote_config.endpoint.value
     remote_url = _blocking.ffi.new("char[]", f"{remote_config_endpoint}".encode('ascii'))
     blocking_config.remote_config.remote_endpoint = remote_url
@@ -156,4 +192,5 @@ def _create_blocking_config():
     blocking_config.remote_config.poll_period_sec = remote_poll_period_seconds
 
     blocking_config.evaluate_body = 1 if traceable_config.blocking_config.evaluate_body.value is True else 0
+    blocking_config.skip_internal_request = 1 if traceable_config.blocking_config.skip_internal_request.value is True else 0 # pylint:disable=C0301
     return blocking_config
